import { PropsWithChildren, useEffect, useRef } from 'react';
import { Dialog, DialogProps } from '../../components/Dialog';
import React from 'react';
import CloseIcon from '@mui/icons-material/Close';
import { useAppDispatch, useQuerybuilderSettings } from '../../data-access';
import { QueryBuilderSettings, setQuerybuilderSettings } from '../../data-access/store/querybuilderSlice';
import { addWarning } from '../../data-access/store/configSlice';

export const QuerySettingsDialog = React.forwardRef<HTMLDivElement, DialogProps>((props, ref) => {
  const qb = useQuerybuilderSettings();
  const dispatch = useAppDispatch();
  const [state, setState] = React.useState<QueryBuilderSettings>(qb);

  useEffect(() => {
    setState(qb);
  }, [qb, props.open]);

  function submit() {
    if (state.depth.min < 0) {
      dispatch(addWarning('The minimum depth cannot be smaller than 0'));
    } else if (state.depth.max > 99) {
      dispatch(addWarning('The maximum depth cannot be larger than 99'));
    } else if (state.depth.min >= state.depth.max) {
      dispatch(addWarning('The minimum depth cannot be larger than the maximum depth'));
    } else {
      dispatch(setQuerybuilderSettings(state));
      props.onClose();
    }
  }

  return (
    <>
      {props.open && (
        <div className="absolute right-20 bottom-5 opacity-100 transition-opacity group-hover:opacity-100 z-50 " ref={ref}>
          <div className="card card-bordered bg-white rounded-none">
            <form
              className="card-body px-0 w-72 py-5"
              onSubmit={(e) => {
                e.preventDefault();
                submit();
              }}
            >
              <div className="card-title p-5 py-0 flex w-full">
                <h2 className="w-full">Query Settings</h2>
                <button className="btn btn-circle btn-sm btn-ghost" onClick={() => props.onClose()}>
                  <CloseIcon fontSize="small" />
                </button>
              </div>
              <div className="divider m-0"></div>
              <div className="form-control px-5">
                <label className="label">
                  <span className="label-text">Limit - Max number of results</span>
                </label>
                <input
                  type="number"
                  className="input input-sm input-bordered"
                  placeholder="500"
                  value={state.limit}
                  onChange={(e) => setState({ ...state, limit: parseInt(e.target.value) })}
                />
              </div>
              <div className="divider m-0"></div>
              <div className="form-control px-5 flex flex-row gap-3">
                <div className="">
                  <label className="label">
                    <span className="label-text">Min Depth Default</span>
                  </label>
                  <input
                    type="number"
                    className="input input-sm input-bordered w-full"
                    placeholder="0"
                    min={0}
                    max={state.depth.max - 1}
                    value={state.depth.min}
                    onChange={(e) => setState({ ...state, depth: { min: parseInt(e.target.value), max: state.depth.max } })}
                    onKeyDown={(e) => {
                      if (e.key === 'Enter') {
                        submit();
                      }
                    }}
                  />
                </div>
                <div className="">
                  <label className="label">
                    <span className="label-text">Max Depth Default</span>
                  </label>
                  <input
                    type="number"
                    className="input input-sm input-bordered w-full"
                    placeholder="0"
                    min={state.depth.min + 1}
                    max={99}
                    value={state.depth.max}
                    onChange={(e) => setState({ ...state, depth: { max: parseInt(e.target.value), min: state.depth.min } })}
                    onKeyDown={(e) => {
                      if (e.key === 'Enter') {
                        submit();
                      }
                    }}
                  />
                </div>
              </div>
              <div className="divider m-0"></div>
              <div className="card-actions mt-1 w-full px-5 flex flex-row">
                <button
                  className="btn btn-secondary flex-grow"
                  onClick={(e) => {
                    e.preventDefault();
                    props.onClose();
                  }}
                >
                  Cancel
                </button>
                <button className="btn btn-primary flex-grow">Apply</button>
              </div>
            </form>
          </div>
        </div>
      )}
    </>
  );
});
