import { useAppDispatch, useML } from '@graphpolaris/shared/lib/data-access';
import {
  setCentralityEnabled,
  setCommunityDetectionEnabled,
  setLinkPredictionEnabled,
  setShortestPathEnabled,
} from '@graphpolaris/shared/lib/data-access/store/mlSlice';

export const QueryBuilderMLPanel = () => {
  const dispatch = useAppDispatch();
  const ml = useML();

  return (
    <div className="card card-bordered w-max rounded-none p-0">
      <div className="card-body px-0 w-72 py-5">
        <div className="p-5 py-0 flex w-full">
          <h2>Machine Learning Options</h2>
        </div>
        <div className="divider m-0"></div>

        <div className="form-control px-5">
          <label className="label cursor-pointer gap-2 w-fit">
            <input
              type="checkbox"
              checked={ml.linkPrediction.enabled}
              className="checkbox checkbox-sm"
              onChange={(e) => dispatch(setLinkPredictionEnabled(e.target.checked))}
            />
            <span className="label-text">Link Prediction</span>
          </label>
          {ml.linkPrediction.enabled && ml.linkPrediction.result && <span># of predictions: {ml.linkPrediction.result.length}</span>}
          {ml.linkPrediction.enabled && !ml.linkPrediction.result && <span>Loading...</span>}
          <label className="label cursor-pointer gap-2 w-fit">
            <input
              type="checkbox"
              checked={ml.centrality.enabled}
              className="checkbox checkbox-sm"
              onChange={(e) => dispatch(setCentralityEnabled(e.target.checked))}
            />
            <span className="label-text">Centrality</span>
          </label>
          {ml.centrality.enabled && Object.values(ml.centrality.result).length > 0 && (
            <span>
              sum of centers:
              {Object.values(ml.centrality.result)
                .reduce((a, b) => b + a)
                .toFixed(2)}
            </span>
          )}
          {ml.centrality.enabled && Object.values(ml.centrality.result).length === 0 && <span>No Centers Found</span>}
          <label className="label cursor-pointer gap-2 w-fit">
            <input
              type="checkbox"
              checked={ml.communityDetection.enabled}
              className="checkbox checkbox-sm"
              onChange={(e) => dispatch(setCommunityDetectionEnabled(e.target.checked))}
            />
            <span className="label-text">Community Detection</span>
          </label>
          {ml.communityDetection.enabled && ml.communityDetection.result && (
            <span># of communities: {ml.communityDetection.result.length}</span>
          )}
          {ml.communityDetection.enabled && !ml.communityDetection.result && <span>Loading...</span>}
          <label className="label cursor-pointer gap-2 w-fit">
            <input
              type="checkbox"
              checked={ml.shortestPath.enabled}
              className="checkbox checkbox-sm"
              onChange={(e) => dispatch(setShortestPathEnabled(e.target.checked))}
            />
            <span className="label-text">Shortest Path</span>
          </label>
          {ml.shortestPath.enabled && ml.shortestPath.result?.length > 0 && <span># of hops: {ml.shortestPath.result.length}</span>}
          {ml.shortestPath.enabled && !ml.shortestPath.srcNode && <span>Please select source node</span>}
          {ml.shortestPath.enabled && ml.shortestPath.srcNode && !ml.shortestPath.trtNode && <span>Please select target node</span>}
        </div>
      </div>
    </div>
  );
};
